Package: CERFIT
Version: 0.2.0
Title: Causal Effect Random Forest of Interaction Trees
Authors@R: c(
    person(given = "Justin",family = "Thorp",email = "jjtthorp@gmail.com",role = c("aut","cre")),
    person(given = "Joshua", family = "Moffat", email = "joshua.moffat@gmail.com", role = "aut"),
    person(given = "Luo",family = "Li",role = "aut"),
    person(given = "Juanjuan", family = "Fan",role = "aut"))
Description: Fits a Causal Effect Random Forest of Interaction Trees (CERFIT) which is a modification of the Random Forest algorithm where each split is chosen to maximize subgroup treatment heterogeneity. Doing this allows it to estimate the individualized treatment effect for each observation in either randomized controlled trial (RCT) or observational data. For more information see L. Li, R. A. Levine, and J. Fan (2022) <doi:10.1002/sta4.457>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
LinkingTo: Rcpp, RcppArmadillo
Imports: partykit, CBPS, randomForest, twang, Rcpp, stats, glmnet,
        survival
Depends: R (>= 2.10)
NeedsCompilation: yes
Packaged: 2026-05-07 14:44:17 UTC; justinthorp
Author: Justin Thorp [aut, cre],
  Joshua Moffat [aut],
  Luo Li [aut],
  Juanjuan Fan [aut]
Maintainer: Justin Thorp <jjtthorp@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-07 16:43:33 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-07 18:53:26 UTC; unix
Archs: CERFIT.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(predict,CERFIT)
S3method(print,CERFIT)
export(CERFIT)
export(MinDepth)
importFrom(Rcpp,sourceCpp)
importFrom(grid,depth)
importFrom(partykit,as.constparty)
importFrom(partykit,fitted_node)
importFrom(partykit,info_node)
importFrom(partykit,kidids_split)
importFrom(partykit,nodeapply)
importFrom(partykit,nodeids)
importFrom(partykit,party)
importFrom(partykit,partynode)
importFrom(partykit,partysplit)
importFrom(partykit,split_node)
importFrom(stats,complete.cases)
importFrom(stats,terms)
useDynLib(CERFIT, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# CERFIT

<!-- badges: start -->
<!-- badges: end -->

The CERFIT R package is an implementation of Random Forest of
Interaction Trees (RFIT) in R. Which is a modification of the random
forest algorithm to estimate individualized treatment effect in
randomized control trials and observational studies. It does this by
modifying the split rule used in the tree growing process. It chooses a
split by maximizing subgroup treatment heterogeneity. It can handle
binary, multiple, ordered, and continuous treatment.

## Installation

You can install the development version of CERFIT from
[GitHub](https://github.com/JustinThorp/CERFIT) with:

``` r
devtools::install_github("JustinThorp/CERFIT")
```

## Example

Below is a small example showing how to fit a Random Forest of
Interaction Trees, get predictions from the tree, and calculate variable
importance. When growing the forest it will print our the response type,
the treatment levels, the type of treatment variable, and an updating
count of the number of trees.

``` r
library(CERFIT)
fit <- CERFIT(Y ~ SAT_MATH + HSGPA + AGE + GENDER + URM | A,
              method = "observation",
              PropForm = "CBPS",
              data = educational,
              ntrees = 30)
#> Continous Response 
#> Treatment Levels: 1 2 3 
#> Multiple Treatment 
#> Tree Number: 10
#> Tree Number: 20
#> Tree Number: 30
ite <- predict(fit)
#> [1] 1 2 3
importance <- MinDepth(fit)
```

``` r
library(tidyverse)
ite %>%
as.data.frame() %>%
pivot_longer(everything()) %>%
ggplot(aes(x = name, y = value)) +
geom_violin() +
scale_x_discrete(labels = c(0,1,2)) +
labs(y = "Predicted Y",x = "A") +
theme_bw()
```

<img src="man/figures/README-ite_plot-1.png" width="100%" />

#' Fits a Random Forest of Interaction Trees
## usethis namespace: start
#' @useDynLib CERFIT, .registration = TRUE
## usethis namespace: end
## usethis namespace: start
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
#' @importFrom partykit partysplit
#' @importFrom partykit party
#' @importFrom partykit partynode
#' @importFrom partykit kidids_split
#' @importFrom partykit nodeids
#' @importFrom partykit fitted_node
#' @importFrom partykit as.constparty
#' @importFrom partykit nodeapply
#' @importFrom partykit split_node
#' @importFrom partykit info_node
#' @importFrom grid depth
#' @importFrom stats complete.cases
#' @importFrom stats terms
#' @description Estimates individualized treatment effects (ITEs) using Random Forest of Interaction Trees.
#' Works with randomized controlled trials (RCTs) and observational data.
#' Treatment variables may be binary, categorical, ordered, or continuous. For binary and survival outcomes, useRes = TRUE must be specified.
#' @param formula Formula to build CERFIT. Categorical predictors must be listed as a factor. e.g., Y ~ x1 + x2 | treatment.
#' For survival outcomes, the response may be specified as Surv(time, event) or Surv(start, stop, event).
#' @param data Data to grow a tree.
#' @param ntrees Number of trees to grow. This value should not be too small, as observations may not be well represented and averaging is insufficient, leading to unstable results. 
#' A value of 1000 is often recommended, but it can be reduced for smaller datasets or to speed up computation.
#' @param subset A logical vector that controls what observations are used to grow the forest.
#' The default value will use the entire data frame.
#' @param search Split search strategy. Options: “exhaustive” (evaluate all cut points) or “sss” (sigmoid approximation, experimental).
#' @param method Study type. "RCT" for randomized data, "observational" for observational data.
#' @param PropForm Method for estimating propensity scores (if method = "observational"). Options: "randomForest", "CBPS", "GBM", "HI". Not all options are compatible with all treatment types. See details.
#' @param split Impurity measure splitting statistic. Currently supports "t.test".
#' @param mtry Number of variables to consider at each split
#' @param nsplit Number of candidate cut points. If NULL, all possible cut points are considered. If an integer is provided, that many cut points are randomly selected from the set of possible cut points.
#' @param nsplit.random Default is FALSE. If TRUE, candidate cut points are chosen randomly.
#' @param minsplit Number of observations required to continue growing tree.
#' @param minbucket Number of observations required in each child node.
#' @param maxdepth Maximum depth of tree.
#' @param oob Logical, whether or not to use out-of-bag sample for predictions. Default is FALSE.
#' @param a Sigmoid approximation variable (for "sss" which is still under development).
#' @param sampleMethod Method to sample learning sample. Default is bootstrap. Subsample
#' takes a subsample of the original data. SubsamplebyID samples by an ID column and
#' uses all observations that have that ID. allData uses the entire data set
#' for every tree.
#' @param useRes Default is TRUE. If TRUE, fits the model using residuals from a regression of the response on covariates (excluding treatment).
#' Improves stability and accuracy. Linear regression is used for continuous responses, logistic regression for binary responses, Cox proportional hazards model are used for survival responses.
#' If response is binary or survival, useRes must be set equal to TRUE.
#' @param scale.y Logical, standardize y when creating splits (For "sss" to increase stability).
#' @param response Response type. Options are "auto", "continuous", "binary", and "survival".
#' When response = auto, the function automatically detects a survival response if the formula uses Surv(...), detects a binary response if the outcome is stored as a factor with exactly two levels, 
#' and otherwise treats the response as continuous. Thus, numeric continuous outcomes are recognized automatically, but binary outcomes coded numerically (for example, 0/1) are not automatically identified as binary
#' and should be specified with response = binary if that behavior is desired.
#' @param surv_resid Residual type used for survival responses. Options are "deviance" and "martingale".
#' @param surv_id Optional subject ID column name for survival models with time-dependent covariates in counting-process format, e.g. Surv(start, stop, event). Required for that format.
#' @return Returns a fitted CERFIT object which is a list containing the following elements:
#' \itemize{
#' \item randFor: The Random Forest of Interaction Trees.
#' \item trt.type: A string containing the treatment type of the data used to fit the model.
#' Can be binary, multiple, ordered, or continuous.
#' \item response.type: A string representing the response type of the data. Can be
#' binary or continuous.
#' \item useRes: A logical indicator that is TRUE if the model was fit on the
#' residuals of a linear model.
#' \item data: The data used to fit the model and includes the estimated propensity score if
#'  method was set to observational.
#' }
#' @details This function implements Random Forest of Interaction Trees proposed
#' in Su (2018), which is a modification of the Random Forest algorithm where
#' instead of a split being chosen to maximize prediction accuracy; each split
#' is chosen to maximize subgroup treatment heterogeneity. It chooses the best
#' split by maximizing the test statistic for \eqn{H_0: \beta_3=0} in the
#' following linear model:
#'
#' \eqn{Y_i = \beta_0 + \beta_1I(X_{ij} < c) + \beta_2I(Z = 1) + \beta_3I(X_{ij} < c)I(Z = 1) + \varepsilon_i}
#'
#' Where \eqn{X_{ij}} represents the splitting variable and Z = 1 represents
#' treatment. So, by maximizing the test statistic for \eqn{\beta_3} we are
#' maximizing the treatment difference between the nodes.
#'
#' The above equation only works when the data comes from a randomized controlled
#' trial, but we can modify it to give us unbiased estimates of treatment
#' effect in observational studies, as shown by Li et al. (2022). To do that we add propensity score into the
#' linear model.
#'
#'\eqn{Y_i = \beta_0 + \beta_1I(X_{ij} < c) + \beta_2I(Z = 1) + \beta_3I(X_{ij} < c)I(Z = 1) + \beta_4e_i + \varepsilon_i}
#'
#'Where \eqn{e_i} represents the propensity score. The CERFIT function will estimate
#'propensity score automatically when the method argument is set to observational.
#'
#'To control how this function estimates propensity score, you can use the
#'PropForm argument. Which can take four possible values randomForest, CBPS,
#' GBM and HI. randomForest uses the randomForest package to use a random forest
#' to estimate propensity score, CBPS uses Covariate Balancing Propensity Score
#' to estimate propensity score, GBM uses generalized boosted regression models
#' to estimate propensity score, and HI is the Hirano–Imbens generalized 
#' propensity score weighting for continuous treatments.
#' Some of these options only work for certain treatment types. 
#' See the full list below.
#' \itemize{
#'  \item binary: GBM, CBPS, randomForest
#'  \item categorical: GBM, CBPS
#'  \item ordered: GBM, CBPS
#'  \item continuous: CBPS, HI
#' }
#' Note: CBPS option supports up to four categories/levels of treatment, 
#' if there are more than four use GBM.
#'
#' @references
#' \itemize{
#' \item Li, Luo, et al. Causal Effect Random Forest of
#' Interaction Trees for Learning Individualized Treatment Regimes with
#'  Multiple Treatments in Observational Studies. Stat, 2022,
#'  https://doi.org/10.1002/sta4.457.
#' \item Su, X., Peña, A., Liu, L., & Levine, R. (2018). Random forests of interaction trees for estimating individualized treatment effects in randomized trials.
#' Statistics in Medicine, 37(17), 2547- 2560.
#' \item G. W. Imbens, The role of the propensity score in estimating dose-response
#' functions., Biometrika, 87 (2000), pp. 706–710.
#' \item G. Ridgeway, D. McCarey, and A. Morral, The twang package: Toolkit for
#' weighting and analysis of nonequivalent groups, (2006).
#' \item A. Liaw and M. Wiener, Classification and regression by randomforest, R
#' News, 2 (2002), pp. 18–22}
#' @examples
#' fit <- CERFIT(Result_of_Treatment ~ sex + age + Number_of_Warts + Area + Time + Type | treatment,
#' data = warts,
#' ntrees = 30,
#' method = "RCT",
#' mtry = 2)
#'
#' @export
### Grows a random forest ###
# Res is for fitting the residuals
CERFIT <- function( formula, data, ntrees, subset = NULL,search=c("exhaustive","sss"),
                    method=c("RCT","observational"), PropForm=c("randomForest","CBPS","GBM", "HI"),
                    split=c("t.test"),
                    mtry=NULL, nsplit=NULL, nsplit.random=FALSE, minsplit=20, minbucket=round(minsplit/3), maxdepth=30,oob = FALSE,
                    a=50, sampleMethod=c('bootstrap','subsample','subsampleByID','allData'),
                    useRes=TRUE, scale.y=FALSE,
                    response=c("auto","continuous","binary","survival"),
                    surv_resid=c("deviance","martingale"),
                    surv_id=NULL)#
{
  sampleMethod <- match.arg(sampleMethod, c('bootstrap','subsample','subsampleByID','allData'))

  response <- match.arg(response, c("auto","continuous","binary","survival"))
  surv_resid <- match.arg(surv_resid, c("deviance","martingale"))
  if (!is.null(surv_id) && !is.character(surv_id)) {
    stop("surv_id must be a character string naming the subject ID column.", call. = FALSE)
  }

  # Detect survival response of the form Surv(time, event) or Surv(start, stop, event)
  lhs <- formula[[2]]
  is_surv <- is.call(lhs) && identical(as.character(lhs[[1]]), "Surv")
  surv_is_counting <- FALSE
  surv_time_vars <- NULL
  surv_event_var <- NULL

  # Build an internal response variable so downstream code can keep using all.vars(formula)[1]
  y_name <- all.vars(formula)[1]
  trt_name <- all.vars(formula)[length(all.vars(formula))]

  if (is_surv) {
    surv_args <- as.list(lhs)[-1]
    n_surv_args <- length(surv_args)

    if (n_surv_args == 2) {
      surv_time_vars <- as.character(surv_args[[1]])
      surv_event_var <- as.character(surv_args[[2]])
    } else if (n_surv_args == 3) {
      surv_time_vars <- c(as.character(surv_args[[1]]), as.character(surv_args[[2]]))
      surv_event_var <- as.character(surv_args[[3]])
      surv_is_counting <- TRUE
      if (is.null(surv_id)) {
        stop("surv_id must be provided when using time-dependent covariates with Surv(start, stop, event).", call. = FALSE)
      }
      if (!surv_id %in% names(data)) {
        stop("surv_id must name a column in data.", call. = FALSE)
      }
    } else {
      stop("Survival responses must be of the form Surv(time, event) or Surv(start, stop, event).", call. = FALSE)
    }

    # Covariates: everything except survival time variables, event indicator, and treatment
    covars <- setdiff(all.vars(formula)[-length(all.vars(formula))], c(surv_time_vars, surv_event_var))

    # Internal response variable name used for residuals
    y_name <- ".CERFIT_y"

    # Rebuild formula as: .CERFIT_y ~ x1 + ... + xp | treatment
    rhs <- if (length(covars) > 0) paste(covars, collapse = " + ") else "1"
    formula <- stats::as.formula(paste(y_name, "~", rhs, "|", trt_name))

    # If user didn't explicitly set response, treat Surv(...) as survival
    if (response == "auto") response <- "survival"
  }

  response.type <- "continuous"

  if (response == "survival") {
    response.type <- "survival"
  } else {
    response_vec <- data[[all.vars(formula)[1]]]
    if (is.factor(response_vec) && length(levels(response_vec)) == 2) {
      response.type <- "binary"
    }
  }

  response_print <- paste0(toupper(substring(response.type,first = 1,last = 1)),
                          substring(response.type,first = 2))
  cat(paste(response_print,"Response","\n"))

  if (useRes) {
    if (response.type == "survival") {
      # For survival: fit Cox model excluding treatment, then use Cox residuals as the working response
      covars <- all.vars(formula)[2:(length(all.vars(formula)) - 1)]
      rhs <- if (length(covars) > 0) paste(covars, collapse = " + ") else "1"

      if (surv_is_counting) {
        start_name <- surv_time_vars[1]
        stop_name <- surv_time_vars[2]
        event_name <- surv_event_var

        resformula <- stats::as.formula(
          paste0("survival::Surv(", start_name, ",", stop_name, ",", event_name, ") ~ ", rhs)
        )
        cox_fit <- survival::coxph(resformula, data = data, id = data[[surv_id]])

        # Preserve original survival outcome components
        data$yo_tstart <- data[[start_name]]
        data$yo_tstop <- data[[stop_name]]
        data$yo_delta <- data[[event_name]]
        data$yo_id <- data[[surv_id]]
      } else {
        time_name <- surv_time_vars[1]
        event_name <- surv_event_var

        resformula <- stats::as.formula(
          paste0("survival::Surv(", time_name, ",", event_name, ") ~ ", rhs)
        )
        cox_fit <- survival::coxph(resformula, data = data)

        # Preserve original survival outcome components
        data$yo_time <- data[[time_name]]
        data$yo_delta <- data[[event_name]]
      }

      # Residual choice: martingale or deviance
      if (surv_resid == "deviance") {
        eres <- stats::resid(cox_fit, type = "deviance")
      } else {
        eres <- stats::resid(cox_fit, type = "martingale")
      }

      # Store working response residuals in the internal response column
      data[[all.vars(formula)[1]]] <- as.numeric(eres)

    } else if (response.type == "binary") {
      resformula <- stats::as.formula(paste(all.vars(formula)[1], paste(all.vars(formula)[2:(length(all.vars(formula)) - 1)], collapse = " + "), sep = " ~ "))
      reslm <- stats::glm(resformula, data, family = stats::binomial)
      eres <- (as.numeric(data[[all.vars(formula)[1]]]) - 1) - stats::fitted(reslm)
      data$yo <- data[[all.vars(formula)[1]]]
      data[[all.vars(formula)[1]]] <- eres

    } else {
      resformula <- stats::as.formula(paste(all.vars(formula)[1], paste(all.vars(formula)[2:(length(all.vars(formula)) - 1)], collapse = " + "), sep = " ~ "))
      reslm <- stats::lm(resformula, data)
      eres <- stats::resid(reslm)
      data$yo <- data[[all.vars(formula)[1]]]
      data[[all.vars(formula)[1]]] <- eres
    }
  } else {
    if (response.type == "survival") {
      # Preserve original survival outcome components for reference
      if (surv_is_counting) {
        data$yo_tstart <- data[[surv_time_vars[1]]]
        data$yo_tstop <- data[[surv_time_vars[2]]]
        data$yo_delta <- data[[surv_event_var]]
        data$yo_id <- data[[surv_id]]
      } else {
        data$yo_time <- data[[surv_time_vars[1]]]
        data$yo_delta <- data[[surv_event_var]]
      }
      # Ensure internal response column exists (set to 0 if not residualizing)
      data[[all.vars(formula)[1]]] <- 0
    } else {
      data$yo <- data[[all.vars(formula)[1]]]
    }
  }
  trt_var <- all.vars(formula)[length(all.vars(formula))]
  TrT <- data[[trt_var]]
  trt_unique <- unique(TrT)
  trt.length <- length(trt_unique)
  if (trt.length < 2) stop("Only one treatment?", call. = FALSE)
  trt.type <- ifelse(trt.length == 2, "binary", "multiple")
  trt.type <- ifelse(is.ordered(TrT), "ordered", trt.type)
  trt.type <- ifelse(is.numeric(TrT) && trt.length > 10, "continuous", trt.type)
  trtlevels <- seq_len(trt.length)
  trttype_print <- paste0(toupper(substring(trt.type,first = 1,last = 1)),
                            substring(trt.type,first = 2))
  cat("Treatment Levels: ")
  cat(paste0(trtlevels),"\n")
  cat(paste(trttype_print,"Treatment","\n"))


  if(method=="observational"){
    propformula <- stats::as.formula(paste(all.vars(formula)[length(all.vars(formula))], paste(all.vars(formula)[2:(length(all.vars(formula))-1)], collapse=" + "), sep=" ~ "))
    if(trt.type=="continuous"){
      if(PropForm=="CBPS"){
        propfun <- CBPS::CBPS(propformula, data = data[,all.vars(formula)[-1]],ATT=FALSE,method = "exact")#
        prop <- propfun$fitted.values
        Iptw <- propfun$weights
      } else if(PropForm=="HI") {
        TrT_num <- as.numeric(TrT)
        propfun <- stats::lm(propformula, data = data[all.vars(formula)[-1]])
        prt <- as.numeric(stats::predict(propfun))
        sigm <- summary(propfun)$sigma
        prop <- stats::dnorm(TrT_num, mean = prt, sd = sigm)
        modhi <- stats::lm(TrT_num ~ 1)
        ps.num <- stats::dnorm(TrT_num, mean = as.numeric(modhi$fitted), sd = summary(modhi)$sigma)
        Iptw <- ps.num / prop
      }
    } else if(trt.type=="binary") {
      if (PropForm=="GBM") {
        propfun <- twang::ps(propformula,data=data[,all.vars(formula)[-1]],interaction.depth = 4, stop.method = "es.max",estimand="ATE",verbose=FALSE,n.trees = 10000)
        prop <- propfun$ps
        Iptw<- twang::get.weights(propfun,stop.method = "es.max",estimand="ATE")
      } else if (PropForm=="CBPS") {
        propfun <- CBPS::CBPS(propformula, data = data[,all.vars(formula)[-1]],ATT=FALSE,method = "exact")#
        prop <- propfun$fitted.values
        Iptw <- propfun$weights
      } else if (PropForm=="randomForest") {
        propfun<- suppressWarnings(randomForest::randomForest(propformula,data=data[all.vars(formula)[-1]]))
        prop <- propfun$predicted
        Iptw <- sum(TrT)/length(TrT)*TrT/prop+sum(1-TrT)/length(TrT)*(1-TrT)/(1-prop)
        #Iptw <-TrT/prop+(1-TrT)/(1-prop)
        Iptw <- truncquant(Iptw[[1]],q=0.9)
      }
    } else if (trt.type=="multiple"){
      if(PropForm=="GBM") {
        data[,all.vars(formula)[length(all.vars(formula))]]<-as.factor(data[,all.vars(formula)[length(all.vars(formula))]])
        propfun <- twang::mnps(propformula,data=data[,all.vars(formula)[-1]],interaction.depth = 4, stop.method = "es.max",estimand="ATE",verbose=FALSE,n.trees = 10000)
        pslist<-propfun$psList
        prop<-matrix(NA,ncol=trt.length,nrow=nrow(data))
        for(i in 1:trt.length){
          prop[,i]<-unlist(pslist[[i]]$ps)
        }
        colnames(prop)<-levels(data[,all.vars(formula)[length(all.vars(formula))]])
        levels(data[,all.vars(formula)[length(all.vars(formula))]])<-c(1:trt.length)
        Iptw <- twang::get.weights(propfun,stop.method = "es.max",estimand="ATE")
      } else if (PropForm=="CBPS" & trt.length<5 ) {
        data[,all.vars(formula)[length(all.vars(formula))]]<-as.factor(data[,all.vars(formula)[length(all.vars(formula))]])
        propfun <- CBPS::CBPS(propformula, data = data[,all.vars(formula)[-1]],ATT=FALSE,method = "exact")#
        prop <- propfun$fitted.values
        Iptw <- propfun$weights
        levels(data[,all.vars(formula)[length(all.vars(formula))]])<-c(1:trt.length)

      }
    } else if(trt.type == "ordered") {
      if(PropForm == "GBM") {
        prop <- matrix(NA,ncol= length(unique(TrT)),nrow=nrow(data))
        propfun <- twang::mnps(propformula,data=data[,all.vars(formula)[-1]],interaction.depth = 4,
                               stop.method = "es.max",estimand="ATE",verbose=FALSE,n.trees = 10000)
        pslist <- propfun$psList
        for(i in 1:length(unique(TrT))){
          prop[,i]<-unlist(pslist[[i]]$ps)
        }
        prop <- t(apply(prop, 1, cumsum))
        prop <- as.data.frame(prop)[,colSums(is.na(prop)) == 0]
        prop <- prop / prop[,length(unique(TrT))]
        #names(prop) <- TrT_splits[!is.na(TrT_splits)]
        Iptw <- twang::get.weights(propfun,stop.method = "es.max",estimand="ATE")
        #Iptw<- rep(1,nrow(data))
        #Iptw <- sum(TrT)/length(TrT)*TrT/prop+sum(1-TrT)/length(TrT)*(1-TrT)/(1-prop)
        #Iptw <-TrT/prop+(1-TrT)/(1-prop)
        Iptw <- truncquant(Iptw,q=0.9)
      } else if (PropForm == "CBPS") {
        #data[,all.vars(formula)[length(all.vars(formula))]]<-as.factor(data[,all.vars(formula)[length(all.vars(formula))]])
        propfun <- CBPS::CBPS(propformula, data = data[,all.vars(formula)[-1]],ATT=FALSE,method = "exact")#
        prop <- propfun$fitted.values
        print(prop)
        Iptw <- propfun$weights
        levels(data[,all.vars(formula)[length(all.vars(formula))]])<-c(1:trt.length)
      } else if (PropForm == "old") {
        prop <- matrix(NA,ncol= length(unique(TrT)),nrow=nrow(data))
        propfun <- twang::mnps(propformula,data=data[,all.vars(formula)[-1]],interaction.depth = 4,
                               stop.method = "es.max",estimand="ATE",verbose=FALSE,n.trees = 10000)
        pslist <- propfun$psList
        for(i in 1:length(unique(TrT))){
          prop[,i]<-unlist(pslist[[i]]$ps)
        }
        prop <- t(apply(prop, 1, cumsum))
        prop <- as.data.frame(prop)[,colSums(is.na(prop)) == 0]
        #names(prop) <- TrT_splits[!is.na(TrT_splits)]
        Iptw <- twang::get.weights(propfun,stop.method = "es.max",estimand="ATE")
        #Iptw<- rep(1,nrow(data))
        #Iptw <- sum(TrT)/length(TrT)*TrT/prop+sum(1-TrT)/length(TrT)*(1-TrT)/(1-prop)
        #Iptw <-TrT/prop+(1-TrT)/(1-prop)
        Iptw <- truncquant(Iptw,q=0.9)
      }
    }  else stop("Please specify a propensity score method: randomForest or CBPS or GBM", call. = FALSE)
  } else if (method=="RCT") {
    prop <- rep(1,nrow(data))#rep("none",nrow(data)) # for observational no prop need
    Iptw<- rep(1,nrow(data))}
  if(!exists("Iptw")) {
    stop("Not able to estimate Propenisty Score. \n Check that your function arguments are correct")
  }
  #data[,all.vars(formula)[length(all.vars(formula))]]<-as.numeric(as.character(data[,all.vars(formula)[length(all.vars(formula))]]))
  data[,all.vars(formula)[-length(all.vars(formula))]] <- sapply(data[,all.vars(formula)[-length(all.vars(formula))]],as.numeric)
  data$iptw <- Iptw
  data$prop <- prop
  if (response.type == "survival") {
    data$surv_resid_type <- surv_resid
    data$surv_is_counting <- surv_is_counting
    if (!is.null(surv_id)) data$surv_id_col <- surv_id
  }
  #data <- cbind(data,prop)
  #return(data)
  #Construct random forest
  randFor <- lapply(1:ntrees,function(b){
    if(b%%10==0){cat(paste0("Tree Number: ",b,"\n"))}
    #print(paste0("Tree Number: ",b))
    obs.b <- switch(sampleMethod,
                    bootstrap = sample.int(nrow(data), size=nrow(data), replace=TRUE, prob=data$iptw), #inverse weighting in boostrapping
                    subsample = sample.int(nrow(data), size=round(nrow(data)*0.632), replace=FALSE,prob=data$iptw), # stratified sampling
                    #subsampleByID = {nIds <- length(unique(data[[idVar]]))
                    #unlist(lapply(sample(unique(data[[idVar]]), size=round(nIds*0.632), replace=FALSE),
                    #              function(x){which(data[[idVar]] == x)}))},
                    allData = 1:nrow(data))
    sample.b <- data[obs.b,]
    if(oob) {oob <- data[-obs.b,]} else {oob <- data[obs.b,]}
    tree.b <- growTree(formula=formula, data=sample.b,oob = oob, subset=subset, search=search, method=method, split=split,
                       mtry=mtry, nsplit=nsplit, nsplit.random=nsplit.random, minsplit=minsplit, minbucket=minbucket, maxdepth=maxdepth, a=a,
                       scale.y=scale.y, useRes=useRes, trtlevels=trtlevels,response.type = response.type)#, useRpart=useRpart, minpvalue=minpvalue, corstr=corstr)
    list(tree=tree.b,cases=sort(unique(obs.b)))
  })
  trt <- data[[all.vars(formula)[length(all.vars(formula))]]]
  #print(length(trt))
  #print(nrow(data))
  #print(length(all.vars(formula)))
  data[[all.vars(formula)[length(all.vars(formula))]]] <- as.numeric(as.character(trt))
  object <- list(randFor = randFor,trt.type = trt.type,
                 response.type = response.type,
                 useRes = useRes,
                 data = data)
  class(object) <- "CERFIT"
  return(object)
}
# Having issues in mutiple treatment where in partition only a single treatment
# is present in the data

#' Calculate Variable Importance
#'
#' @param cerfit A fitted CERFIT object
#' @return Returns a named vector with the name of each predictor used to fit the CERFIT
#' object and its corresponding average minimal depth across all trees.
#' @description Calculates the average minimal depth of each predictor used to fit
#' a CERFIT object. It calculates a variable's importance by using the variable's average minimal depth.
#' Variables with a lower average minimal depth are more important.
#' @details  The depth of the root node is zero and if a variable does not appear
#' at any split in a tree it is assigned maxdepth + 1 for that tree.
#' @examples
#' fit <- CERFIT(Result_of_Treatment ~ sex + age + Number_of_Warts + Area + Time + Type | treatment,
#' data = warts,
#' ntrees = 30,
#' method = "RCT",
#' mtry = 2)
#' importance <- MinDepth(fit)
#' @export
MinDepth <- function(cerfit){  # need to given number of levels if observation
  cerfit <- cerfit$randFor
  Term<-cerfit[[1]]$tree$terms
  dataTemp<-all.vars(Term[[3]])
  vars<-dataTemp[-length(dataTemp)]

  mindepth <- rep(0, length(vars))
  for (t in seq_along(cerfit)) {
    intNodes <- nodeids(cerfit[[t]]$tree)[-nodeids(cerfit[[t]]$tree, terminal = TRUE)]
    varsInTree <- vars[unique(unlist(nodeapply(cerfit[[t]]$tree, ids=intNodes, FUN=function(n){split_node(n)$varid})))]
    varsAtNode <- unlist(nodeapply(cerfit[[t]]$tree, ids=intNodes, FUN=function(n){split_node(n)$varid}))
    #Root node should be counted as 0
    #depthAtNode <- table(unlist(lapply(intNodes, function(x) intersect(intNodes, nodeids(cerfit[[t]]$tree, from=x)))))-1
    #depthAtNode <- idDepth(cerfit[[t]]$tree)
    depthAtNode <- unlist(nodeapply(cerfit[[t]]$tree, ids = intNodes, info_node)) - 2
    treeDepth <- depth(cerfit[[t]]$tree)

    for (j in seq_along(vars)) {
      if (is.element(vars[j], varsInTree)) { #If variable is in tree
        mindepth[j]=mindepth[j]+min(depthAtNode[varsAtNode==j])
      } else {  #If variable not in tree, set mindepth to maximum depth+1
        mindepth[j]=mindepth[j]+treeDepth+1
      }
    }
  }
  mindepth <- mindepth/length(cerfit)
  names(mindepth) <- vars
  return(mindepth)
}

### Convert factors to numerical value. ###
ordinalize <- function(x, y, sortCat=TRUE){
  if (is.factor(x)) {
    x <- factor(x) #Remove factors not listed
    #One can randomly assign a category a distinct numerical value
    if (!sortCat) {
      cutToLvl <- t(sample.int(length(levels(x))))
      colnames(cutToLvl)=levels(x)
    } else {
      #For binary, sort data by proportion in class 1.  For continuous, sort by means
      if (is.factor(y)) {
        cutToLvl <- prop.table(table(y,x),2)[1,,drop=FALSE]
      } else {cutToLvl <- t(vapply(levels(x), function(z){mean(y[x==z])}, numeric(1)))}
    }
    #Convert lvls to numerical value. Slow method. Make this faster later.
    xTemp <- rep(NA,length(x))
    for (lvls in levels(x)) {
      xTemp[x==lvls] <- cutToLvl[colnames(cutToLvl)==lvls]
    }
  } else {
    xTemp <- x
    cutToLvl <- NULL
  }
  return(list(x=xTemp, cutToLvl=cutToLvl))
}

partition<- function(vars, y, trt, propensity, subset, search, method, split, nsplit, nsplit.random,
                     minsplit, minbucket, a, scale.y, useSearch, useOptim,trtlevels,response.type){#, allVars
  if (sum(subset) < minsplit) {return(NULL)}
  vars <- vars[subset,,drop=FALSE]
  y <- y[subset]
  trt<- trt[subset]
  if (length(unique(trt)) < 2) {return(NULL)}
  
  
  # if(length(trtlevels) > 2 & length(trtlevels) < 10 & method != "RCT") { # 10/7/25 JM fix typo & !is.ordered(trt)) {
  #   propensity <- propensity[subset,]
  # } else {
  #   propensity <- propensity[subset]
  # }
  
  # --- Normalize shape of 'propensity' before subsetting (handles GBM vector/data.frame/matrix cases)
  if (is.data.frame(propensity)) {
    # If GBM output was wrapped in a 1-column data.frame, flatten it
    if (ncol(propensity) == 1L) {
      propensity <- as.numeric(propensity[[1L]])
    } else {
      # Multi-arm GBM (multiple treatment levels) → convert to matrix
      propensity <- as.matrix(propensity)
    }
  } else if (is.matrix(propensity) && ncol(propensity) == 1L) {
    # 1-column matrix → flatten to numeric vector
    propensity <- as.numeric(propensity[, 1L])
  }
  
  # --- Subset correctly depending on treatment structure
  if (length(trtlevels) > 2 & length(trtlevels) < 10 & method != "RCT") {
    # multi-arm observational → subset rows + all columns
    propensity <- propensity[subset, , drop = FALSE]
  } else {
    # binary or RCT → subset rows only
    propensity <- propensity[subset]
  }
  
  
  trt.length <- length(trtlevels)
  trt_num_unique <- length(unique(trt))
  is_continuous_trt <- is.numeric(trt) && !is.factor(trt) && !is.ordered(trt) && trt_num_unique > 10
  
  if (method != "RCT") {
    if (is.ordered(trt) || is_continuous_trt) {
      # Treat ordered and numeric-continuous treatments the same way during split evaluation
      trt_num <- as.numeric(trt)
      trt_vals <- sort(unique(trt_num))
      if (length(trt_vals) < 2) {return(NULL)}
      ran <- sample(seq_len(length(trt_vals) - 1L), 1L)
      cutoff_trt <- trt_vals[ran]
      trt <- ifelse(trt_num <= cutoff_trt, 1, 0)

      # For ordered multi-level GPS matrices, align propensity with the temporary dichotomization
      if (is.matrix(propensity) && ncol(propensity) > 1L) {
        propensity <- rowSums(propensity[, seq_len(ran), drop = FALSE])
      }

    } else if (trt.length > 2 & trt.length < 10) {
      ## if less than 10 treatments/levels
      ran <- sample(unique(trt), 2)
      keep <- trt == ran[1] | trt == ran[2]
      vars <- vars[keep, , drop = FALSE]
      y <- y[keep]
      if (is.matrix(propensity)) {
        propensity <- propensity[keep, , drop = FALSE]
      } else {
        propensity <- propensity[keep]
      }
      trt <- trt[keep]
      trt <- ifelse(trt == ran[1], 1, 0)

      # If multi-arm and matrix, grab matching column
      if (is.matrix(propensity)) {
        col_id <- match(as.character(ran[1]), colnames(propensity))
        if (is.na(col_id)) {
          suppressWarnings(col_id <- as.integer(ran[1]))
        }
        if (is.na(col_id) || col_id < 1L || col_id > ncol(propensity)) {
          stop("Could not align propensity column with sampled treatment level")
        }
        propensity <- propensity[, col_id]
      }
    }
  }
  else {
    if (is.ordered(trt) || is_continuous_trt) {
      # Chooses the split point for ordered or numeric-continuous treatment
      trt_num <- as.numeric(trt)
      trt_vals <- sort(unique(trt_num))
      if (length(trt_vals) < 2) {return(NULL)}
      ran <- sample(seq_len(length(trt_vals) - 1L), 1L)
      cutoff_trt <- trt_vals[ran]
      trt <- ifelse(trt_num <= cutoff_trt, 1, 0)
    } else if (trt.length > 2 & trt.length < 10) {
      ## if less than 10 treatments/levels
      ran <- sample(unique(trt), 2)
      keep <- trt == ran[1] | trt == ran[2]
      vars <- vars[keep, , drop = FALSE]
      y <- y[keep]
      trt <- trt[keep]
      trt <- ifelse(trt == ran[1], 1, 0)
    }
  }


  if (NROW(vars) < 2*minbucket) {return(NULL)}
  if (length(unique(y))==1) {return(NULL)}
  stats<- cutoff<- breakLeft<-NA
  findStats<-sapply(vars,function(x){
    x_is_factor <- is.factor(x)
    x_levels <- if (x_is_factor) levels(x) else NULL
    if (search=="exhaustive" && !is.null(nsplit) && nsplit.random) {
      xTemp <- ordinalize(x, y, sortCat=FALSE)
    } else {
      xTemp <- ordinalize(x, y, sortCat=TRUE)
      }
    x <- xTemp$x
    #If all x values the same, do not check optimal split
    if (abs(max(x) - min(x)) > 1e-8) {
      #The SSS partition deals with problems when there is a very small number of observations
      #Use exhaustive search in this case (or set minsplit >= 5)
      if (search=="sss") { #leave sss here for now
        print("sss not ready")
      } else if (search=="exhaustive") { #current codes only work exhaustive search
        cutpts <- findCutpts(x, minbucket)
        #z <- matrix(x,ncol = length(x))[rep(1, length(cutpts)), ] < cutpts
        if (is.null(nsplit)) {
          nsplit<- length(cutpts)
        }
        #Take nsplit cutpoints (if applicable)
        if (!is.null(nsplit) && !is.null(cutpts) && length(cutpts) > 1) {
          #If nsplit.random is TRUE, take nsplit cutpts randomly.  Otherwise, take nsplit cutpts equally spread out across cutpts
          if (!nsplit.random & length(cutpts) > nsplit) { #if not random select nsplit cut
            cutpts <- unique(cutpts[seq(1, length(cutpts), length.out=nsplit)])
          } else {
            cutpts <- sort(sample(cutpts, min(c(nsplit, length(cutpts))), replace=FALSE))
          }
        }
        #
        #It is possible (unlikely) no cutpoint can satisfy minbucket
        if (!is.null(cutpts)) {
          #print(list(y,x,trt,cutpts,method,propensity,minbucket,response.type))
          mod <- find_split(y=y, x=x, trt=trt,cutpts=cutpts,
                            method=method,propensity = propensity,
                            minbucket=minbucket,response_type = response.type)
          if (!is.na(mod$stat)) {
            stats <- mod$stat
            if (x_is_factor) {
              cutoff <- "factor"
              breakLeft <- rep(NA_integer_, length(x_levels))
              names(breakLeft) <- x_levels
              left_lvls <- colnames(xTemp$cutToLvl)[xTemp$cutToLvl <= mod$cutoff]
              right_lvls <- colnames(xTemp$cutToLvl)[xTemp$cutToLvl > mod$cutoff]
              breakLeft[x_levels %in% left_lvls] <- 1L
              breakLeft[x_levels %in% right_lvls] <- 2L
              if (all(is.na(breakLeft)) || length(unique(stats::na.omit(breakLeft))) <= 1) {
                stop("Did not find correct cutpoints")
              }
            }
            else {cutoff <- mod$cutoff; breakLeft<-NA}
          }
        }

      }
      else {stop("Unexpected search")}
    }
    return(c(stats,cutoff,breakLeft))})
  #If randomly picking a subset of categories, do not sort by mean.  Would be more likely to select variables when sorted
  #return(findStats)
  #If each candidate variable cannot be split (e.g. cannot satisfy minbucket), return null
  if (all(is.na(findStats[1,]))) {return(NULL)}
  best_idx <- which.max(findStats[1,])
  if (identical(findStats[2, best_idx], "factor")) {
    # Index is used for categorical variable splits
    return(partysplit(varid = as.integer(colnames(findStats)[best_idx]),
                      index = as.integer(findStats[3, best_idx]),
                      info = list(stats = findStats[1, ])))
  } else {

    #Breaks is used for continuous variable splits
    #print(as.integer(colnames(findStats)[which.max(findStats[1,])]))
    return(partysplit(varid = as.integer(colnames(findStats)[best_idx]),
                      breaks = findStats[2, best_idx],
                      info = list(stats = findStats[1, ])))
  }
}

#' Get predictions from a CERFIT object
#'
#' @param object A fitted CERFIT object
#' @param newdata New data to make predictions from. If not provided will make predictions
#' on training data.
#' @param gridval For continuous treatment. Controls which values of treatment to predict.
#' @param prediction Return prediction using all trees ("overall") or using first i trees ("by iter").
#' @param type Choose which value you wish to predict: 'response' will predict the potential outcome.
#' 'ITE' will predict the individualized treatment effect. And 'opT'
#' will predict the optimal treatment for each observation.
#' @param alpha For continuous treatment. It is the mixing parameter for the elastic
#' net regularization in each node. When equal to 0 it is ridge regression and
#' when equal to 1 it is lasso regression.
#' @param ... Additional Arguments
#' @return The return value depends of the type argument. If type is 'response' the function
#' will return a matrix with n rows and the number of columns equal to the level of treatment.
#' If type is 'ITE' then it returns a matrix with n rows and a number of columns equal to
#' one minus the levels of treatment. And if type is 'opT' then it returns a matrix with n
#' rows and two columns. With the first column denoting the optimal treatment and
#' the second column denoting the optimal response.
#' @examples
#' fit <- CERFIT(Result_of_Treatment ~ sex + age + Number_of_Warts + Area + Time + Type | treatment,
#' data = warts,
#' ntrees = 30,
#' method = "RCT",
#' mtry = 2)
#' ite <- predict(fit,type = "ITE")
#' @export
predict.CERFIT <- function(object,newdata = NULL, gridval=NULL,
                           prediction=c("overall","by iter"),
                           type=c("response","ITE","node","opT"),
                           alpha=0.5,...){

  #Return prediction using all trees ("overall") or using first i trees ("by iter")S
  prediction <- match.arg(prediction, c("overall","by iter"))
  useRse <- object$useRes
  data <- object$data
  if (is.null(newdata)) newdata <- object$data

  # Coerce newdata columns to match the stored training-data classes
  common_vars <- intersect(names(newdata), names(data))
  for (v in common_vars) {
    train_col <- data[[v]]
    new_col <- newdata[[v]]

    if ((is.integer(train_col) || is.numeric(train_col)) && is.factor(new_col)) {
      tmp <- suppressWarnings(as.numeric(as.character(new_col)))
      if (!any(is.na(tmp) & !is.na(new_col))) {
        if (is.integer(train_col)) {
          newdata[[v]] <- as.integer(tmp)
        } else {
          newdata[[v]] <- tmp
        }
      }
    } else if ((is.factor(train_col) || is.ordered(train_col)) && !is.factor(new_col)) {
      newdata[[v]] <- factor(new_col, levels = levels(train_col), ordered = is.ordered(train_col))
    } else if (is.character(train_col) && !is.character(new_col)) {
      newdata[[v]] <- as.character(new_col)
    } else if (is.logical(train_col) && !is.logical(new_col)) {
      newdata[[v]] <- as.logical(new_col)
    } else if (is.numeric(train_col) && !is.numeric(new_col) && !is.factor(new_col)) {
      newdata[[v]] <- as.numeric(new_col)
    } else if (is.integer(train_col) && !is.integer(new_col) && !is.factor(new_col)) {
      newdata[[v]] <- as.integer(new_col)
    }
  }

  # Note: "node" is accepted for forward compatibility but is not yet implemented.
  # It is excluded from the package documentation. If developed in the future,
  # a corresponding return block should be added in the prediction == "overall"
  # and prediction == "by iter" sections below.
  
  response.type <- object$response.type
  treatment.type <- object$trt.type
  object <- object$randFor
  type <- match.arg(type, c("response","ITE","node","opT"))
  cumMeanNA <- function(x){
    xTemp <- x;
    xTemp[is.na(xTemp)] <- 0
    cumsum(xTemp)/cumsum(!is.na(x))
    }
  #utrt<- sort(unique(c(fitted(x[[1]]$tree)[,3],fitted(x[[2]]$tree)[,3],fitted(x[[3]]$tree)[,3])))
  formulaTree <- stats::formula(object[[1]]$tree$terms)
  treatment <- all.vars(formulaTree)[length(all.vars(formulaTree))]
  utrt<-sort(unique(data[[treatment]]))
  LB<-min(data[[treatment]])
  UB<-max(data[[treatment]])
  qu<-seq(LB,UB,length.out = 6)
  ## add a statement warning if gridvalue beyond the LB and UB
  ## should add warnings here if gridbalue beyond min or max utrt
  ntrt <- length(utrt)
  # if grival is null, use the 10th quantile
  if(useRse == TRUE & response.type == "continuous"){
    resformula <-  stats::as.formula(paste("yo", paste(all.vars(formulaTree)[2:(length(all.vars(formulaTree))-1)], collapse=" + "), sep=" ~ "))
    reslm <- stats::lm(resformula,data)
    ylmp <- stats::predict(reslm,newdata)
    #print("WHAT")
  } else if(useRse == TRUE & response.type == "binary") {
    resformula <-  stats::as.formula(paste("yo", paste(all.vars(formulaTree)[2:(length(all.vars(formulaTree))-1)], collapse=" + "), sep=" ~ "))
    reslm <- stats::glm(resformula,data,family = stats::binomial())
    ylmp <- stats::predict(reslm,newdata,type = "response")
    print(length(ylmp))
  } else {
    ylmp<-rep(0,nrow(newdata))
  }
  if(length(utrt)<=15){ ## if less than or equal to 15 unique treatments/levels using unique treatments
    ntrt=length(utrt)
    gridval<-utrt
  } else if(is.null(gridval)) { # if more than 15, and gridval is null, use percentiles at 5% increment
    gridval <- stats::quantile(utrt, prob = seq(0, 1, length = 21))
    ntrt<-length(gridval)-1
  } else {
    ntrt<-length(gridval)}
  print(gridval)

  if(type!="opT"){
    predictMat <- lapply(lapply(object, "[[" , "tree"), predictTree, newdata=newdata,gridval=gridval,ntrt=ntrt,type=type,LB=LB,UB=UB,alpha=alpha)
    ypre<- do.call(cbind,predictMat)
    #yp<- lapply(1:ntrt,function(i,k) k[,seq(i, by = ntrt, length = NCOL(ypre) / ntrt)],k=ypre)
    ypre<- lapply(1:ntrt,function(i,k) k[,seq(i, NCOL(ypre), by = ntrt)], k=ypre)
    y.pre<- t(matrix(unlist(lapply(ypre,rowMeans,na.rm=TRUE)), ncol=NROW(newdata),byrow = TRUE))
    y.pre<-y.pre+ylmp
    #y.pre: by row observation, each column is the corresponding predition for 1 treatment.
  } else if (type == "opT" && treatment.type != "continuous"){
    predictMat<-lapply(lapply(object , "[[" , "tree"), predictTree, newdata=newdata,gridval=gridval,ntrt=ntrt,type="opT",  LB=LB,UB=UB,alpha=alpha)
    #ntrt<-2
    ypre<- do.call(cbind,predictMat)
    ypre<- lapply(1:ntrt,function(i,k) k[,seq(i, NCOL(ypre), by = ntrt)], k=ypre)
    y.pre<- t(matrix(unlist(lapply(ypre,rowMeans,na.rm=TRUE)), ncol=NROW(newdata),byrow = TRUE))
    y.pre<- y.pre + ylmp
    t.opt <- max.col(y.pre)
    y.opt <- apply(y.pre, 1, max, na.rm = TRUE)
    #topt<-as.matrix(ypre[[1]])
    #yopt<-as.matrix(ypre[[2]])
    #y.opt<-rowMeans(yopt,na.rm = T)+ylmp
    #t.opt<-rowMeans(topt,na.rm = T)
    y.pre<- cbind(t.opt,y.opt)
  } else {
    predictMat<-lapply(lapply(object , "[[" , "tree"), predictTree, newdata=newdata,gridval=gridval,ntrt=ntrt,type="opT",  LB=LB,UB=UB,alpha=alpha)
    ntrt<-2
    ypre<- do.call(cbind,predictMat)
    ypre<- lapply(1:ntrt,function(i,k) k[,seq(i, NCOL(ypre), by = ntrt)], k=ypre)
    topt<-as.matrix(ypre[[1]])
    yopt<-as.matrix(ypre[[2]])
    y.opt<-rowMeans(yopt)+ylmp
    t.opt<-rowMeans(topt)
    y.pre<- cbind(t.opt,y.opt)
  }

  yname<-NA
  if (prediction=="overall") {
    if(type=="response") {
      resp <- y.pre
      yname<- paste("y=",gridval,sep="")
      colnames(resp) <- yname
      return(resp)}
    if(type=="ITE") { #using the first level or smallest value as reference group
      yname <- paste("y", gridval[-1], "-y", gridval[1], sep = "")
      ite <- y.pre - y.pre[,1]
      ite <- ite[,-1, drop = FALSE]
      colnames(ite) <- yname
      return(ite)
    }
    
    if(type=="opT") {
      yname<-c("opTreat","opResponse")
      opTY<-y.pre
      colnames(opTY) <- c(yname)
      return(opTY)
    }
  }
  else if(prediction=="by iter"){
    Ypre<-as.list(NA)
    for(i in 1: ntrt){
      Ypre[[i]]<-t(apply(ypre[[i]],1,cumMeanNA))
    }
    cumypre<-t(matrix(unlist(Ypre),ncol=NROW(newdata),byrow = TRUE))
    ntree<-length(object)
    cumypre.l<- lapply(seq(1,(ntrt*ntree),by=ntree),function(i,k) k[,i:(i+ntree-1)], k=cumypre)
    print(cumypre.l)
    if(type=="response"){
      yname<-paste("ycum",utrt,sep="")
      names(cumypre.l) <- yname
      return(cumypre.l)}
    if(type=="ITE")  {
      cumite<-as.list(NA)
      for(i in 1:ntrt){
        cumite[[i]]<- cumypre.l[[i]]-cumypre.l[[1]]
      }
      yname<-paste("ycum",utrt,"-ycum",utrt[1],sep="")
      names(cumite)<-yname
      print(yname)
      return(cumite[[-1]])
    }}

}

#' @export
print.CERFIT <- function(x,...){
  cat(paste("Numer of Trees:",length(x$randFor),"\n"))
  cat(paste("Treatment Type:",x$trt.type, "\n"))
  cat(paste("Response Type:",x$response.type))
}
# CapStr <- function(y) {
#   c <- strsplit(y, " ")[[1]]
#   paste(toupper(substring(c, 1,1)), substring(c, 2),
#         sep="", collapse=" ")
# }

growTree <- function(formula, data,oob, subset=NULL, search=c("exhaustive","sss"),
                     method=c("RCT","observational"),
                     split=c("t.test", "pvalue"),#, "gini", "entropy", "information"),
                     mtry=NULL, nsplit=NULL, nsplit.random=TRUE, minsplit=20, minbucket=round(minsplit/3), maxdepth=20,
                     a=50, useRes, scale.y=FALSE, trtlevels,response.type)
{
  search <- match.arg(search,c("exhaustive","sss"))
  method <- match.arg(method,c("RCT","observational"))
  split <- match.arg(split,c("t.test", "pvalue"))
  stopifnot(is.logical(nsplit.random), is.logical(scale.y), is.logical(useRes))#, is.logical(useRpart))
  if (is.numeric(nsplit) && !nsplit.random && nsplit < 5) {"Selecting <5 ordered splits may yield unexpected results"}
  response <- all.vars(formula)[1]
  if(grepl("\\|", as.character(formula)[3])){
    treatment <- trimws(strsplit(as.character(formula)[3], "\\|")[[1]][2], which="both")
  } else {stop("Please specify the treatment in formula")}

  Propensity <- "prop"
  Iptw <- "iptw"
  data <- data[c(all.vars(formula)[-1], response, Iptw,Propensity)] #Rearrange data so that response comes last
  oob <- oob[c(all.vars(formula)[-1], response, Iptw,Propensity)]

  if (!all(complete.cases(data[-length(data)])) & !is.null(subset)) { paste0("Specifying subset with missing data can yield unexpected results") }
  data <- data[complete.cases(data[-length(data)]),]
  # The reason for not checking the last column for NA's is that the last column can be a
  # data.frame(?) and would through an error. Since the last column is propensity any
  # NA's are caused by NA is other columns so this should be fine


  if (is.null(mtry)){mtry <- length(all.vars(formula[[3]]))-1} #defult mtry=p

  #if(is.factor(data[[response]])){data[[response]]=as.numeric(data[[response]]==levels(data[[response]])[1])}

  if (is.null(subset)){subset <- rep(TRUE, nrow(data))}

  # Grow tree
  nodes <- growTemp(id=1L, depth=1L, data=data, response=response, treatment=treatment, Propensity=Propensity, subset=subset, search=search, method=method, split=split,
                    mtry=mtry, nsplit=nsplit, nsplit.random=nsplit.random, minsplit=minsplit, minbucket=minbucket,
                    maxdepth=maxdepth, a=a, scale.y=scale.y, trtlevels=trtlevels,response.type = response.type)

  # Compute terminal node number for each observation
  fitted <- fitted_node(nodes, data=oob)

  thing <- ncol(oob[Propensity])
  if(thing !=1) {
    daprop<-cbind(oob[[treatment]],oob[[Propensity]])
    ps<-apply(daprop,1,function(v){x<-v[1];return(v[x+1])})
  }
  ps<- oob[[Propensity]]
  # Return rich constparty object
  ret <- party(nodes, data = oob,
               fitted = data.frame("(fitted)" = fitted,
                                   "(response)" = oob[[response]],
                                   "(treatment)" = oob[[treatment]],
                                   "(propensity)"= ps,#data[[Propensity]],
                                   "(iptw)"= oob[[Iptw]],
                                   check.names = FALSE),
               terms = terms(formula))
  as.constparty(ret)
  #as.simpleparty(ret)
}

#' Observational Educational Dataset
#'
#' A simulated dataset containing the grades and other attributes of 1000
#' simulated students.
#'
#'
#' @format A data frame with 1000 rows and 7 variables:
#' \describe{
#'   \item{SAT_MATH}{SAT Math Score}
#'   \item{HSGPA}{High School GPA}
#'   \item{AGE}{Age of Student}
#'   \item{GENDER}{Gender of Student}
#'   \item{URM}{Under Represented Minority}
#'   \item{A}{Treatment Variable}
#'   \item{Y}{Students Final Grade}
#' }
#' @source Wilke, Morten C., et al. “Estimating the Optimal Treatment
#' Regime for Student Success Programs.” Behaviormetrika, vol. 48, no. 2, 2021,
#' pp. 309–343., https://doi.org/10.1007/s41237-021-00140-0.
"educational"


#' Randomized Controlled Trial Warts Dataset
#'
#' A dataset comparing immunotherapy to cryotherapy treatments and their effectiveness of
#' removing warts.
#'
#'
#' @format A data frame with 180 rows and 8 variables:
#' \describe{
#'   \item{sex}{Patients Sex}
#'   \item{age}{Patients Age}
#'   \item{Time}{Time Elapsed Before Treatment}
#'   \item{Number_of_Warts}{Number of Warts}
#'   \item{Type}{Type of Wart}
#'   \item{Area}{Wart Surface Area}
#'   \item{Result_of_Treatment}{Treatment Outcome}
#'   \item{treatment}{0 for immunotherapy and 1 for cryotherapy}
#' }
#' @source Khozeimeh, Fahime, et al. “An Expert System for Selecting Wart
#' Treatment Method.” Computers in Biology and Medicine, vol. 81, 2017,
#' pp. 167–175., https://doi.org/10.1016/j.compbiomed.2017.01.001.
"warts"

### Return predicted values from a tree. ###
predictTree <- function(tree, newdata=tree$data, gridval, LB, UB, ntrt,type="response",alpha ){
  parse_tree_formula <- function(form) {
    form_chr <- paste(deparse(form), collapse = " ")
    form_chr <- gsub("\\s+", " ", form_chr)

    split_tilde <- strsplit(form_chr, "~", fixed = TRUE)[[1]]
    if (length(split_tilde) != 2) {
      stop("Unable to parse fitted CERFIT tree formula.", call. = FALSE)
    }

    rhs_full <- trimws(split_tilde[2])
    rhs_parts <- strsplit(rhs_full, "|", fixed = TRUE)[[1]]
    if (length(rhs_parts) != 2) {
      stop("Unable to identify predictors from fitted CERFIT tree formula.", call. = FALSE)
    }

    x_part <- trimws(rhs_parts[1])
    trt_part <- trimws(rhs_parts[2])

    predictors <- trimws(unlist(strsplit(x_part, "\\+")))
    predictors <- predictors[nzchar(predictors)]

    list(
      predictors = predictors,
      treatment = trt_part
    )
  }
  da <- stats::fitted(tree)
  colnames(da)[2:5]<-c("y","Trt","prop","ww")
  ufit<-sort(unique(da[["(fitted)"]]))

  formulaTree <- stats::formula(tree$terms)
  formula_info <- parse_tree_formula(formulaTree)
  predictors <- formula_info$predictors
  treatment <- formula_info$treatment

  required_cols <- unique(c(predictors, treatment))
  if (!all(required_cols %in% colnames(newdata))) {
    missing_cols <- setdiff(required_cols, colnames(newdata))
    stop(
      paste(
        "newdata is missing columns required for tree prediction:",
        paste(missing_cols, collapse = ", ")
      ),
      call. = FALSE
    )
  }
  newdata_tree <- newdata[, required_cols, drop = FALSE]

  tree_tmp <- tree
  tree_tmp$terms <- stats::terms(
    stats::as.formula(
      paste("~", paste(required_cols, collapse = " + "))
    )
  )
  nodesNewdata <- stats::predict(tree_tmp, newdata=newdata_tree, type="node")
  if (length(nodesNewdata) != nrow(newdata_tree)) {
    stop("Tree node assignment failed for one or more rows in newdata.", call. = FALSE)
  }
  Y.min<-ifelse(min(da[,2])<0,2*min(da[,2]),min(da[,2])/2)
  Y.max<-ifelse(max(da[,2])<0,max(da[,2])/2,2*max(da[,2]))
  #if (ntrt<=10){ #for binary and multiple trt, ignore gridval
  if(ntrt<=10){
    pred<- lapply(split(da ,list(da[["(fitted)"]],da[,3])), function(da){
      ytemp<-try(stats::weighted.mean(da[,2],da[,length(da)],na.rm=T))#This is only using the first propensity (fixed)
      if(inherits(ytemp,"try-error")) {
        return(NA)
        } else {
          return(ytemp)
          }
    })
    nodepred<- cbind(ufit,t(matrix(unlist(pred), ncol = length(ufit), byrow = TRUE)))
  } else {
    pred<- lapply(split(da ,da[["(fitted)"]]), function(da){
      Trt<-da$Trt
      x<-cbind(Trt,Trt^2,Trt^3)
      lambdas <- 10^seq(5, -3, by = -.1)
      fit <- try(glmnet::cv.glmnet(x, da$y,  family = "gaussian", alpha = alpha, lambda = lambdas,nfolds =10),silent=TRUE)
      if (inherits(fit, "try-error")){
        fit2<-try(glmnet::cv.glmnet(x, jitter(da$y), family = "gaussian", alpha = alpha, lambda = lambdas, nfolds =10),silent=TRUE)
        if (inherits(fit2, "try-error")) {
          return(NA)} else {fit<-fit2}
      }

      Trt<-gridval
      ext<-Trt>max(da[,3])|Trt<min(da[,3])
      nd<-cbind(gridval,gridval^2,gridval^3)
      ytemp <- stats::predict(fit, newx = nd, s=fit$lambda.min)
      ytemp[!ext]=ifelse(ytemp[!ext]>Y.max,Y.max,ytemp[!ext])##avoid extrem values
      ytemp[!ext]=ifelse(ytemp[!ext]<Y.min,Y.min,ytemp[!ext])#mean(da[,2])
      ytemp[ext]=ifelse(ytemp[ext]>Y.max,NA,ytemp[ext])
      ytemp[ext]=ifelse(ytemp[ext]<Y.min,NA,ytemp[ext])
      if (type!="opT") {
        return(ytemp)
      }else {
        #top<-gridval[which.max(ytemp)]
        #yop<-max(ytemp)
        lengthout<-5
        B <- seq(LB, UB, length.out=lengthout)
        opY<-Y.min; opTrt <- NA
        pref<-function(Trt){
          trtt<-cbind(Trt,Trt^2,Trt^3)
          yp<- stats::predict(fit, newx = trtt, s=fit$lambda.min)
          return(yp)}
        for (b in 1:(lengthout-1)) {
          fit.tmp <-  suppressWarnings(stats::optimize(pref, lower=B[b], upper=B[b+1], maximum=TRUE))
          if (is.na(fit.tmp$objective)) {
            opY<-opY
            opTrt<-opTrt
          } else {
            if (!is.nan(fit.tmp$objective) && fit.tmp$objective > opY && fit.tmp$objective < Y.max ) {
              opY <- fit.tmp$objective
              opTrt <- fit.tmp$maximum
            }
          } }
        return(cbind(opTrt,opY))}
    })
    nodepred<- cbind(ufit,matrix(unlist(pred), ncol = length(pred[[1]]), byrow = TRUE))
  }
  if(type=="opT" && ntrt  > 10) {
    ntrt<-2
    }
  predictions<-as.data.frame(cbind(nodesNewdata,matrix(NA,ncol=ntrt,nrow=nrow(newdata))))
  matched_rows <- match(predictions$nodesNewdata,nodepred[,1])
  if (nrow(nodepred) == 0 || all(is.na(matched_rows))) {
    stop("No terminal-node predictions could be matched to newdata in predictTree().", call. = FALSE)
  }
  predictions[,2:(ntrt+1)] <- nodepred[matched_rows,2:(ntrt+1), drop = FALSE]
  return(predictions[,2:(ntrt+1)])
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

find_split <- function(y, x, trt, cutpts, method, propensity, minbucket, response_type) {
    .Call(`_CERFIT_find_split`, y, x, trt, cutpts, method, propensity, minbucket, response_type)
}

### Grow tree by using partition() function several times in a recursive loop ###
growTemp <- function(id=1L, depth=1L, data, response, treatment, Propensity, subset, search, method, split,
                     mtry, nsplit, nsplit.random, minsplit, minbucket, maxdepth,
                     a, scale.y, trtlevels,response.type){
  if (depth > maxdepth) {return(partynode(id=id))}
  y <- data[[response]]
  trt<-data[[treatment]]
  propensity<-data[[Propensity]]
  varSelected <- sort(sample.int(ncol(data)-4, mtry))
  vars <- data[varSelected]
  colnames(vars) <- varSelected #Have columns represent varid

  sp <- partition(vars=vars, y=y,  subset=subset,trt=trt,propensity=propensity,
                  search=search, method=method, split=split, nsplit=nsplit, nsplit.random=nsplit.random,
                  minsplit=minsplit, minbucket=minbucket, a=a, scale.y=scale.y,
                  trtlevels=trtlevels,response.type = response.type)
  #useSearch=useSearch, useOptim=useOptim,
  if (is.null(sp)) {return(partynode(id=id))}

  # Split the data
  kidids <- kidids_split(sp, data=data)
  depth <- depth + 1
  #print(max(kidids, na.rm=TRUE))
  kids <- vector(mode="list", length=max(kidids, na.rm=TRUE))
  for (kidid in seq_along(kids)) {
    s <- subset # subset is the previous loops s
    s[kidids != kidid] <- FALSE
    # Node ID
    if (kidid > 1) {myid <- max(nodeids(kids[[kidid-1]]))
    } else {myid <- id}
    # Start recursion on this daugther node
    kids[[kidid]] <- growTemp(id=as.integer(myid+1), depth=depth, data=data, response=response, treatment=treatment, Propensity=Propensity,
                              subset=s, search=search, method=method, split=split, mtry=mtry, nsplit=nsplit, nsplit.random=nsplit.random,
                              minsplit=minsplit, minbucket=minbucket, maxdepth=maxdepth,
                              a=a, scale.y=scale.y, trtlevels=trtlevels,response.type = response.type)
  }
  #print(sapply(kids, class))
  #print(length(kids))
  #print(class(kids))
  return(partynode(id=as.integer(id), split=sp, kids=kids,
                   #info=list(stats=max(info_split(sp)$stats, na.rm=TRUE))))
                   info = depth))
}

###Finds candidate cutpoints ###
findCutpts <- function(x, minbucket) {
  nX <- length(x)
  x_sort <- sort(x,method = "quick")
  #Due to ties, it's possible minbucket cannot be satisfied
  if (x_sort[minbucket]==rev(x_sort)[minbucket]) {
    cutpts=NULL}
  else {
    #Cutpoints considered must satisfy minbucket
    cutpts <- unique(x_sort[minbucket:(nX-minbucket+1)])

    if(length(cutpts)==1){stop(paste0("Only 1 cutpt??? ", cutpts, x))}
    cutpts <- (cutpts[1:(length(cutpts)-1)]+cutpts[2:length(cutpts)])/2
  }
  return(cutpts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CERFIT.R
\name{CERFIT}
\alias{CERFIT}
\title{Fits a Random Forest of Interaction Trees}
\usage{
CERFIT(
  formula,
  data,
  ntrees,
  subset = NULL,
  search = c("exhaustive", "sss"),
  method = c("RCT", "observational"),
  PropForm = c("randomForest", "CBPS", "GBM", "HI"),
  split = c("t.test"),
  mtry = NULL,
  nsplit = NULL,
  nsplit.random = FALSE,
  minsplit = 20,
  minbucket = round(minsplit/3),
  maxdepth = 30,
  oob = FALSE,
  a = 50,
  sampleMethod = c("bootstrap", "subsample", "subsampleByID", "allData"),
  useRes = TRUE,
  scale.y = FALSE,
  response = c("auto", "continuous", "binary", "survival"),
  surv_resid = c("deviance", "martingale"),
  surv_id = NULL
)
}
\arguments{
\item{formula}{Formula to build CERFIT. Categorical predictors must be listed as a factor. e.g., Y ~ x1 + x2 | treatment.
For survival outcomes, the response may be specified as Surv(time, event) or Surv(start, stop, event).}

\item{data}{Data to grow a tree.}

\item{ntrees}{Number of trees to grow. This value should not be too small, as observations may not be well represented and averaging is insufficient, leading to unstable results.
A value of 1000 is often recommended, but it can be reduced for smaller datasets or to speed up computation.}

\item{subset}{A logical vector that controls what observations are used to grow the forest.
The default value will use the entire data frame.}

\item{search}{Split search strategy. Options: “exhaustive” (evaluate all cut points) or “sss” (sigmoid approximation, experimental).}

\item{method}{Study type. "RCT" for randomized data, "observational" for observational data.}

\item{PropForm}{Method for estimating propensity scores (if method = "observational"). Options: "randomForest", "CBPS", "GBM", "HI". Not all options are compatible with all treatment types. See details.}

\item{split}{Impurity measure splitting statistic. Currently supports "t.test".}

\item{mtry}{Number of variables to consider at each split}

\item{nsplit}{Number of candidate cut points. If NULL, all possible cut points are considered. If an integer is provided, that many cut points are randomly selected from the set of possible cut points.}

\item{nsplit.random}{Default is FALSE. If TRUE, candidate cut points are chosen randomly.}

\item{minsplit}{Number of observations required to continue growing tree.}

\item{minbucket}{Number of observations required in each child node.}

\item{maxdepth}{Maximum depth of tree.}

\item{oob}{Logical, whether or not to use out-of-bag sample for predictions. Default is FALSE.}

\item{a}{Sigmoid approximation variable (for "sss" which is still under development).}

\item{sampleMethod}{Method to sample learning sample. Default is bootstrap. Subsample
takes a subsample of the original data. SubsamplebyID samples by an ID column and
uses all observations that have that ID. allData uses the entire data set
for every tree.}

\item{useRes}{Default is TRUE. If TRUE, fits the model using residuals from a regression of the response on covariates (excluding treatment).
Improves stability and accuracy. Linear regression is used for continuous responses, logistic regression for binary responses, Cox proportional hazards model are used for survival responses.
If response is binary or survival, useRes must be set equal to TRUE.}

\item{scale.y}{Logical, standardize y when creating splits (For "sss" to increase stability).}

\item{response}{Response type. Options are "auto", "continuous", "binary", and "survival".
When response = auto, the function automatically detects a survival response if the formula uses Surv(...), detects a binary response if the outcome is stored as a factor with exactly two levels,
and otherwise treats the response as continuous. Thus, numeric continuous outcomes are recognized automatically, but binary outcomes coded numerically (for example, 0/1) are not automatically identified as binary
and should be specified with response = binary if that behavior is desired.}

\item{surv_resid}{Residual type used for survival responses. Options are "deviance" and "martingale".}

\item{surv_id}{Optional subject ID column name for survival models with time-dependent covariates in counting-process format, e.g. Surv(start, stop, event). Required for that format.}
}
\value{
Returns a fitted CERFIT object which is a list containing the following elements:
\itemize{
\item randFor: The Random Forest of Interaction Trees.
\item trt.type: A string containing the treatment type of the data used to fit the model.
Can be binary, multiple, ordered, or continuous.
\item response.type: A string representing the response type of the data. Can be
binary or continuous.
\item useRes: A logical indicator that is TRUE if the model was fit on the
residuals of a linear model.
\item data: The data used to fit the model and includes the estimated propensity score if
method was set to observational.
}
}
\description{
Estimates individualized treatment effects (ITEs) using Random Forest of Interaction Trees.
Works with randomized controlled trials (RCTs) and observational data.
Treatment variables may be binary, categorical, ordered, or continuous. For binary and survival outcomes, useRes = TRUE must be specified.
}
\details{
This function implements Random Forest of Interaction Trees proposed
in Su (2018), which is a modification of the Random Forest algorithm where
instead of a split being chosen to maximize prediction accuracy; each split
is chosen to maximize subgroup treatment heterogeneity. It chooses the best
split by maximizing the test statistic for \eqn{H_0: \beta_3=0} in the
following linear model:

\eqn{Y_i = \beta_0 + \beta_1I(X_{ij} < c) + \beta_2I(Z = 1) + \beta_3I(X_{ij} < c)I(Z = 1) + \varepsilon_i}

Where \eqn{X_{ij}} represents the splitting variable and Z = 1 represents
treatment. So, by maximizing the test statistic for \eqn{\beta_3} we are
maximizing the treatment difference between the nodes.

The above equation only works when the data comes from a randomized controlled
trial, but we can modify it to give us unbiased estimates of treatment
effect in observational studies, as shown by Li et al. (2022). To do that we add propensity score into the
linear model.

\eqn{Y_i = \beta_0 + \beta_1I(X_{ij} < c) + \beta_2I(Z = 1) + \beta_3I(X_{ij} < c)I(Z = 1) + \beta_4e_i + \varepsilon_i}

Where \eqn{e_i} represents the propensity score. The CERFIT function will estimate
propensity score automatically when the method argument is set to observational.

To control how this function estimates propensity score, you can use the
PropForm argument. Which can take four possible values randomForest, CBPS,
GBM and HI. randomForest uses the randomForest package to use a random forest
to estimate propensity score, CBPS uses Covariate Balancing Propensity Score
to estimate propensity score, GBM uses generalized boosted regression models
to estimate propensity score, and HI is the Hirano–Imbens generalized
propensity score weighting for continuous treatments.
Some of these options only work for certain treatment types.
See the full list below.
\itemize{
\item binary: GBM, CBPS, randomForest
\item categorical: GBM, CBPS
\item ordered: GBM, CBPS
\item continuous: CBPS, HI
}
Note: CBPS option supports up to four categories/levels of treatment,
if there are more than four use GBM.
}
\examples{
fit <- CERFIT(Result_of_Treatment ~ sex + age + Number_of_Warts + Area + Time + Type | treatment,
data = warts,
ntrees = 30,
method = "RCT",
mtry = 2)

}
\references{
\itemize{
\item Li, Luo, et al. Causal Effect Random Forest of
Interaction Trees for Learning Individualized Treatment Regimes with
Multiple Treatments in Observational Studies. Stat, 2022,
https://doi.org/10.1002/sta4.457.
\item Su, X., Peña, A., Liu, L., & Levine, R. (2018). Random forests of interaction trees for estimating individualized treatment effects in randomized trials.
Statistics in Medicine, 37(17), 2547- 2560.
\item G. W. Imbens, The role of the propensity score in estimating dose-response
functions., Biometrika, 87 (2000), pp. 706–710.
\item G. Ridgeway, D. McCarey, and A. Morral, The twang package: Toolkit for
weighting and analysis of nonequivalent groups, (2006).
\item A. Liaw and M. Wiener, Classification and regression by randomforest, R
News, 2 (2002), pp. 18–22}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minDepth.R
\name{MinDepth}
\alias{MinDepth}
\title{Calculate Variable Importance}
\usage{
MinDepth(cerfit)
}
\arguments{
\item{cerfit}{A fitted CERFIT object}
}
\value{
Returns a named vector with the name of each predictor used to fit the CERFIT
object and its corresponding average minimal depth across all trees.
}
\description{
Calculates the average minimal depth of each predictor used to fit
a CERFIT object. It calculates a variable's importance by using the variable's average minimal depth.
Variables with a lower average minimal depth are more important.
}
\details{
The depth of the root node is zero and if a variable does not appear
at any split in a tree it is assigned maxdepth + 1 for that tree.
}
\examples{
fit <- CERFIT(Result_of_Treatment ~ sex + age + Number_of_Warts + Area + Time + Type | treatment,
data = warts,
ntrees = 30,
method = "RCT",
mtry = 2)
importance <- MinDepth(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{warts}
\alias{warts}
\title{Randomized Controlled Trial Warts Dataset}
\format{
A data frame with 180 rows and 8 variables:
\describe{
\item{sex}{Patients Sex}
\item{age}{Patients Age}
\item{Time}{Time Elapsed Before Treatment}
\item{Number_of_Warts}{Number of Warts}
\item{Type}{Type of Wart}
\item{Area}{Wart Surface Area}
\item{Result_of_Treatment}{Treatment Outcome}
\item{treatment}{0 for immunotherapy and 1 for cryotherapy}
}
}
\source{
Khozeimeh, Fahime, et al. “An Expert System for Selecting Wart
Treatment Method.” Computers in Biology and Medicine, vol. 81, 2017,
pp. 167–175., https://doi.org/10.1016/j.compbiomed.2017.01.001.
}
\usage{
warts
}
\description{
A dataset comparing immunotherapy to cryotherapy treatments and their effectiveness of
removing warts.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.CERFIT}
\alias{predict.CERFIT}
\title{Get predictions from a CERFIT object}
\usage{
\method{predict}{CERFIT}(
  object,
  newdata = NULL,
  gridval = NULL,
  prediction = c("overall", "by iter"),
  type = c("response", "ITE", "node", "opT"),
  alpha = 0.5,
  ...
)
}
\arguments{
\item{object}{A fitted CERFIT object}

\item{newdata}{New data to make predictions from. If not provided will make predictions
on training data.}

\item{gridval}{For continuous treatment. Controls which values of treatment to predict.}

\item{prediction}{Return prediction using all trees ("overall") or using first i trees ("by iter").}

\item{type}{Choose which value you wish to predict: 'response' will predict the potential outcome.
'ITE' will predict the individualized treatment effect. And 'opT'
will predict the optimal treatment for each observation.}

\item{alpha}{For continuous treatment. It is the mixing parameter for the elastic
net regularization in each node. When equal to 0 it is ridge regression and
when equal to 1 it is lasso regression.}

\item{...}{Additional Arguments}
}
\value{
The return value depends of the type argument. If type is 'response' the function
will return a matrix with n rows and the number of columns equal to the level of treatment.
If type is 'ITE' then it returns a matrix with n rows and a number of columns equal to
one minus the levels of treatment. And if type is 'opT' then it returns a matrix with n
rows and two columns. With the first column denoting the optimal treatment and
the second column denoting the optimal response.
}
\description{
Get predictions from a CERFIT object
}
\examples{
fit <- CERFIT(Result_of_Treatment ~ sex + age + Number_of_Warts + Area + Time + Type | treatment,
data = warts,
ntrees = 30,
method = "RCT",
mtry = 2)
ite <- predict(fit,type = "ITE")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{educational}
\alias{educational}
\title{Observational Educational Dataset}
\format{
A data frame with 1000 rows and 7 variables:
\describe{
\item{SAT_MATH}{SAT Math Score}
\item{HSGPA}{High School GPA}
\item{AGE}{Age of Student}
\item{GENDER}{Gender of Student}
\item{URM}{Under Represented Minority}
\item{A}{Treatment Variable}
\item{Y}{Students Final Grade}
}
}
\source{
Wilke, Morten C., et al. “Estimating the Optimal Treatment
Regime for Student Success Programs.” Behaviormetrika, vol. 48, no. 2, 2021,
pp. 309–343., https://doi.org/10.1007/s41237-021-00140-0.
}
\usage{
educational
}
\description{
A simulated dataset containing the grades and other attributes of 1000
simulated students.
}
\keyword{datasets}
